#include "decl.h"

int calc_dt(double *dt_receiver,double GPS_secs,struct ephemeris_record GPS_eph,
                        double approx_position[3],struct GNSS_record GPS_record)

     // This function calculates the receiver clock offset from GPS code
     // measurement, satellite clock offset, and propagation delay.
     // See GPS SPS Signal Specification, AD 04-03
     //
     // Input  GPS_secs        : nominal time
     //        GPS_eph         : last GPS ephemerides
     //        approx_position : RIMS station position from control file
     //        GPS_record      : last GPS measurements
     //
     // Output dt_receiver     : receiver clock offset
     //        calc_dt         : (boolean) dt_receiver value valid (1/0)

{
  double c=299792458.0;
  double Omega_dot_e=7.2921151467e-5;  
  double t_oc_f,sta_x,sta_y,sta_z,xcorr,ycorr,sat_coord[3];
  double dt_sat,dt_r,dt_propagation,dt_preliminary,dt_candidates[NSAT];
  int i,j,n_sat,ok_dt=0,n_hits[NSAT];
  double dt_lim=2.0e-7;
  int n_neigbors_required=3;


  // Reset the counts how many other "dt candidates"
  // that "fairly well" agree with dt_candidates[i]
  for (i=0; i < NSAT; i++) n_hits[i]=0;

  // station coordinates
  sta_x=approx_position[0];
  sta_y=approx_position[1];
  sta_z=approx_position[2];

  // Loop over all possible GPS satellites to find a receiver dt value 
  // that agrees well for at least n_neigbors_required=3 satellites
  for (i=0; i < NSAT; i++)
    {
    // check if P1(C1) and ephemerides exists
    if ((GPS_record.status[i]&4)&&(GPS_eph.eph_status[i]))
      {
      t_oc_f=GPS_eph.t_oc[i]+7.0*86400.0*GPS_eph.week_n[i];
      // compensate for possible week roll over problems
      if (t_oc_f > GPS_secs+3.5*86400.0) t_oc_f = t_oc_f - 7.0*86400.0;
      if (t_oc_f < GPS_secs-3.5*86400.0) t_oc_f = t_oc_f + 7.0*86400.0;

      dt_r=calc_dt_relativistic(GPS_secs,GPS_eph,i+1);

      // calculate the satellite clock offset
      dt_sat=GPS_eph.a_f0[i] + GPS_eph.a_f1[i]*(GPS_secs - t_oc_f) +
             dt_r - GPS_eph.t_gd[i];

      // Iterate the signal propagation delay calculations twice
      // to adjust satellite coordinates for large receiver offsets
      // during the second turn
      dt_preliminary=0;
      for (j=0; j < 2; j++)
        {
        get_sat_coord(sat_coord,GPS_secs+dt_preliminary,GPS_eph,i+1);
        dt_propagation=(pow((sat_coord[0]-sta_x)*(sat_coord[0]-sta_x)+
                            (sat_coord[1]-sta_y)*(sat_coord[1]-sta_y)+
                            (sat_coord[2]-sta_z)*(sat_coord[2]-sta_z),0.5))/c;

        // Correction for coordinates in an Earth-fixed (non-inertial) system
        xcorr=-Omega_dot_e*sta_y*dt_propagation;
        ycorr= Omega_dot_e*sta_x*dt_propagation;
 
        // Estimate receiver clock offset from (free space) signal propagation
        // delay, receiver code measurement, and known satellite clock offset
        // (No atmospheric delay compenasation implemented)
        dt_preliminary=(pow(
             (sat_coord[0]-sta_x-xcorr)*(sat_coord[0]-sta_x-xcorr)+
             (sat_coord[1]-sta_y-ycorr)*(sat_coord[1]-sta_y-ycorr)+
             (sat_coord[2]-sta_z      )*(sat_coord[2]-sta_z      ),0.5) -
                            GPS_record.P1[i])/c - dt_sat;
       }

      dt_candidates[i]=dt_preliminary;

      // Compare the new dt candidate with the previous ones, (j<i),
      // and mark those that don't deviate "much" (dt_lim) from the new candidate
      for (j=0; j < i; j++) if ((GPS_eph.eph_status[j])&&
             (fabs(dt_candidates[j]-dt_candidates[i])<dt_lim)) n_hits[j]++;

      }

    }  

  // Find a dt candidate which is within dt_lim for at least n_neigbor_required
  // (i.e. n_hits at least n_neigbors_required-1 "hits").
  // If not found dt_ok remains = 0
  for (i=0; i < NSAT; i++)
    if (n_hits[i]>=n_neigbors_required-1)
      {                        
      *dt_receiver=dt_candidates[i];
      ok_dt=1;
      }

  return ok_dt;

}
