#include "decl.h"

struct GNSS_record decode_GNSS(struct GNSS_record old_record,int system_type,
                                  int n_sats_obs,unsigned char in_bytes[max_bytes])

     // This function decodes a section of RIMS raw measurement bytes and
     // flags the status (what type of measurements are valid) for each satellite.
     // See EGNOS CCF Interface Control Document (CCF ICD) for section structure.
     //
     // Input  old_record  : last measurement record for this system type
     //        system_type : 0=GPS, 1=GLONASS, 2=GEO measurement
     //        n_sats_obs  : # satellite measurements to decode
     //        in_bytes    : RIMS file bytes to decode
     //
     // Output decode_GNSS : new measurement record

{
  struct GNSS_record new_record;
  int i,sat,L1_val,L2_val,P_val;
  int n_bytes_per_sat,gps_offset_bytes,sat_numbering_offset;
  int decoding_error=0;
  double pseudo_range,diff_pseudo_range=0;

  switch(system_type)
    {
    case 0:                  // GPS
      n_bytes_per_sat=32;
      gps_offset_bytes=2;
      sat_numbering_offset=0;
      break;
    case 1:                  // GLONASS
      n_bytes_per_sat=22;
      gps_offset_bytes=0;
      sat_numbering_offset=37;
      break;
    case 2:                  // GEO
      n_bytes_per_sat=32;
      gps_offset_bytes=0;
      sat_numbering_offset=100;
      break;
    }

  for (i=0; i < NSAT; i++) new_record.status[i] = 0;

  for (i=0; i < n_sats_obs; i++)
    if (!decoding_error)
    {
    sat=(int)in_bytes[n_bytes_per_sat*i]-sat_numbering_offset;
    if ((sat<1)||(sat>NSAT))
      decoding_error=1;      // satellite number out of range
    else
      {
      // map validity values to a common status value:
      //L1_val L2_val P_val status
      //  0     0      0     0
      //  1     0      0     1
      //  0     1      0     2
      //  1     1      0     3
      //  0     0      1     4
      //  1     0      1     5
      //  0     1      1    10
      //  1     1      1    15
      L1_val=!((int)in_bytes[3+gps_offset_bytes+n_bytes_per_sat*i]&64);
      if (system_type==0) L2_val=!((int)in_bytes[3+gps_offset_bytes+n_bytes_per_sat*i]&32);
      else L2_val=0;
      P_val=!((int)in_bytes[ 3+gps_offset_bytes+n_bytes_per_sat*i]&128);
      new_record.status[sat-1]=1*L1_val+2*L2_val+4*P_val+4*P_val*L2_val*(1+L1_val);
    
      new_record.L1[sat-1]=decode_phase(in_bytes+9+gps_offset_bytes+n_bytes_per_sat*i,
  				       old_record.status[sat-1]&1,old_record.L1[sat-1]);
      new_record.C_N0_L1[sat-1]=(int)in_bytes[13+gps_offset_bytes+n_bytes_per_sat*i];
      pseudo_range=decode_pseudo_range(in_bytes+4+gps_offset_bytes+n_bytes_per_sat*i);

      if (system_type==0) {
        new_record.L2[sat-1]=decode_phase(in_bytes+17+gps_offset_bytes+n_bytes_per_sat*i,
				         old_record.status[sat-1]&2,old_record.L2[sat-1]);
        new_record.C_N0_L2[sat-1]=(int)in_bytes[21+gps_offset_bytes+n_bytes_per_sat*i]; 
        diff_pseudo_range=decode_diff_pseudo_range(in_bytes+15+gps_offset_bytes+n_bytes_per_sat*i); }

      if (L2_val&&!L1_val)
        new_record.P2[sat-1]=pseudo_range;
      else
        {
        new_record.P1[sat-1]=pseudo_range;
        new_record.P2[sat-1]=pseudo_range+diff_pseudo_range;
        }
      }
    }
    
  // If any decoding error occured mark ALL satellites measurements as "not valid"
  if (decoding_error) for (i=0; i < NSAT; i++) new_record.status[i] = 0;

  return new_record;
}

