#include "decl.h"

struct measurement_record extract_measurements(struct measurement_record old_measurements,
                                           unsigned char in_bytes[max_bytes],int gps_only)

     // This function read the GNSS measurement sections contained in the 
     // RIMS raw measurement message, and extracts their data.
     // See EGNOS CCF Interface Control Document (CCF ICD) for 
     // RIMS raw measurement message and GNSS measurement sections.
     // See EGNOS CCF Interface Control Document (CCF ICD) Annex 2
     // for details on the RIMS data types.
     //
     // Input  old_measurements     : last useful measurement record
     //        in_bytes             : RIMS raw measurement message bytes
     //        gps_only             : 1 if only GPS written to rinex file
     //
     // Output extract_measurements : new_measurement record

{
  struct measurement_record new_measurements;
  double secs;
  int i,section_type,n_sats,stop_decoding=0,ok_new_data=0;
  int message_length,section_start=header_length,section_length=0;

  // reset the new measurement status values
  new_measurements=reset_data();

  // Extract the present GPS time from the EGNOS Std Data header
  secs=1024.0*7.0*86400.0;
  for (i=0; i < 6; i++) secs=secs+(double)in_bytes[i+13]*pow(256.0,(double)(3-i));
  new_measurements.GPS_secs=(double)((long)(secs));
  if (new_measurements.GPS_secs < old_measurements.GPS_secs) stop_decoding=1;

  // read length of RIMS raw measurement message
  // (we know from before that its length is at least header_length+3)
  message_length=256*(int)in_bytes[3]+(int)in_bytes[4];
  for (i=0; i < 3; i++) 
    {
    section_start+=section_length;
    // check that section don't point beyond the message length
    if (section_start+3<message_length)
      {
      section_type=(int)in_bytes[section_start];
      section_length=256*(int)in_bytes[section_start+1]+
                         (int)in_bytes[section_start+2];
      }
    else stop_decoding=1;
    // check that decoding has been ok so far, that the section_length don't point 
    // beyond the message length, and that the section type is one of interest 
    if ((!stop_decoding)&&(section_start+section_length<=message_length)&&
                 (!((section_type!=11)&&(gps_only))))
      switch(section_type)
	{
        case 11:       // GPS
          // derive number of satellites in the data from the section length
          n_sats=(section_length-3)/32;
          // check if number of satellites within boundaries,
          // and that the section length fits perfectly with the 
          // derived number of satellites
          if ((n_sats>0)&&(n_sats<=NSAT)&&(n_sats*32==section_length-3))
            new_measurements.GPS=decode_GNSS(old_measurements.GPS,0,n_sats,
                                             in_bytes+section_start+3);
          break;
        case 12:       // GLONASS
          // derive number of satellites in the data from the section length
          n_sats=(section_length-3)/22;
          // check if number of satellites within boundaries,
          // and that the section length fits perfectly with the 
          // derived number of satellites
          if ((n_sats>0)&&(n_sats<=NSAT)&&(n_sats*22==section_length-3))
            new_measurements.GLONASS=decode_GNSS(old_measurements.GLONASS,1,n_sats,
                                               in_bytes+section_start+3);
          break;
        case 13:       // GEO
          // derive number of satellites in the data from the section length
          n_sats=(section_length-3)/32;
          // check if number of satellites within boundaries,
          // and that the section length fits perfectly with the 
          // derived number of satellites
          if ((n_sats>0)&&(n_sats<=NSAT)&&(n_sats*32==section_length-3))
            new_measurements.GEO=decode_GNSS(old_measurements.GEO,2,n_sats,
                                             in_bytes+section_start+3);
          break;
        }
    }

  // Check for any new writeable data, given state of "gps_only"
  for (i=0; i < NSAT; i++) if ((new_measurements.GPS.status[i]!=0)||
                ((!gps_only)&&((new_measurements.GLONASS.status[i]!=0)||
                               (new_measurements.GEO.status[i]!=0))))
    ok_new_data=1;

  // Now, arrange the final output
  if (ok_new_data)
    {
    // mark that there are new writeable data
    new_measurements.writeable=1;
    return new_measurements;
    }
  else 
    {
    // mark that there are no new writeable data
    old_measurements.writeable=0;
    // return last useful record again, to be used in later 
    // calls to decode_GNSS
    return old_measurements;
    }
}

