#include "decl.h"

int extract_station_data(char rin_head[18][81],char rinex_sta[4],double c_n0_lim[6],double appr_pos[3],
                    int samp_interv,int gps_only,char run_by_name[20],char rims_sta[5])

     // This function read the station control file and 
     // creates a rinex file header template.
     // See "RINEX: The Receiver Independent Exchange Format Version 2.10"
     // by Werner Gurtner, AIUB, for definitions.
     //
     // Input  samp_interv          : rinex file sampling tim interval in seconds
     //        gps_only             : (boolean) 1 if only GPS data in the rinex file
     //        run_by_name          : program user name for the rinex file
     //        rims_sta             : RIMS station token
     //
     // Output rin_head             : rinex file header template
     //        rinex_sta            : standard rinex file name beginning
     //        c_n0_lim             : rinex snr value mapping limits
     //        appr_pos             : approximate station coordinates
     //        extract_station_data : (boolean) 1 if reading ok

{
  int read_ok=1,station_found=0,file_is_open=0;
  char in_string[120],name_test_string[7];
  double in_dbl;
  FILE *fd_ctl;

  // create the most general rinex file header template
  strcpy(rin_head[ 0],"     2.10           OBSERVATION DATA                        RINEX VERSION / TYPE\0");
  strcpy(rin_head[ 1],"rims2rinex v2.0                                          UTCPGM / RUN BY / DATE \0");
  strcpy(rin_head[ 2],"BIT 2 OF LLI FLAGS DATA COLLECTED UNDER A/S CONDITION       COMMENT             \0");
  strcpy(rin_head[ 3],"                                                            MARKER NAME         \0");
  strcpy(rin_head[ 4],"                                                            MARKER NUMBER       \0");
  strcpy(rin_head[ 5],"                                                            OBSERVER / AGENCY   \0");
  strcpy(rin_head[ 6],"                                                            REC # / TYPE / VERS \0");
  strcpy(rin_head[ 7],"                                                            ANT # / TYPE        \0");
  strcpy(rin_head[ 8],"                                                            APPROX POSITION XYZ \0");
  strcpy(rin_head[ 9],"                                                            ANTENNA: DELTA H/E/N\0");
  strcpy(rin_head[10],"     1     1                                                WAVELENGTH FACT L1/2\0");
  strcpy(rin_head[11],"     4    L1    L2    C1    P2                              # / TYPES OF OBSERV \0");
  strcpy(rin_head[12],"                                                            INTERVAL            \0");
  strcpy(rin_head[13]," C/N0 is mapped to RINEX snr flag value [1-9]               COMMENT             \0");
  strcpy(rin_head[14],"  L1:     -> 1;     -> 5;     -> 9                          COMMENT             \0");
  strcpy(rin_head[15],"  L2:     -> 1;     -> 5;     -> 9                          COMMENT             \0");
  strcpy(rin_head[16],"                                                GPS         TIME OF FIRST OBS   \0");
  strcpy(rin_head[17],"                                                            END OF HEADER       \0");

  // update template with "observation type" information, program user, and sampling interval
  if (gps_only) strncpy(rin_head[ 0]+40,"G (GPS)",7);
  else          strncpy(rin_head[ 0]+40,"M (MIXED)",9);
  strncpy(rin_head[ 1]+20,run_by_name,strlen(run_by_name));
  sprintf(rin_head[12],"%10.3f",(double)samp_interv);
  // overwrite the null character with a blank
  strncpy(rin_head[12]+10," ",1);

  if ((fd_ctl=fopen(control_file_name,"r")) == NULL)     {
    fprintf(stderr, "Cannot open control file: %s%s",control_file_name,lin_end); read_ok=0;}
  else file_is_open=1;

  // make a control file station search token, "* xyyyy" where xyyyy is rims_sta
  strcpy(name_test_string,"* ");
  strncat(name_test_string,rims_sta,5);
  // read control file line by line to find the station search token
  if (read_ok)
    while ((!station_found)&&(read_ok))  
      {
      if (fgets(in_string,120,fd_ctl)!= NULL) 
        station_found=(!strncmp(in_string,name_test_string,7));
      else 
        {
        fprintf(stderr, "Cannot find station %s in control file %s%s",
                                          rims_sta,control_file_name,lin_end); 
        read_ok=0;}
      }

  if (read_ok) if (read_ok=read_ctl_str(rinex_sta,"RINEX NAME:",4,1,fd_ctl)) 

  // read the other 9 string elements of the control file,
  // and copy them to the header template
  if (read_ok) if (read_ok=read_ctl_str(in_string,"MARKER NAME:",60,2,fd_ctl)) 
                  strncpy(rin_head[ 3],in_string,60);
  if (read_ok) if (read_ok=read_ctl_str(in_string,"MARKER NUMBER:",60,3,fd_ctl)) 
                  strncpy(rin_head[ 4],in_string,60);
  if (read_ok) if (read_ok=read_ctl_str(in_string,"OBSERVER:",20,4,fd_ctl)) 
                  strncpy(rin_head[ 5],in_string,20);
  if (read_ok) if (read_ok=read_ctl_str(in_string,"AGENCY:",40,5,fd_ctl)) 
                  strncpy(rin_head[ 5]+20,in_string,40);
  if (read_ok) if (read_ok=read_ctl_str(in_string,"REC #:",20,6,fd_ctl)) 
                  strncpy(rin_head[ 6],in_string,20);
  if (read_ok) if (read_ok=read_ctl_str(in_string,"REC TYPE:",20,7,fd_ctl)) 
                  strncpy(rin_head[ 6]+20,in_string,20);
  if (read_ok) if (read_ok=read_ctl_str(in_string,"REC VERS:",20,8,fd_ctl)) 
                  strncpy(rin_head[ 6]+40,in_string,20);
  if (read_ok) if (read_ok=read_ctl_str(in_string,"ANT #:",20,9,fd_ctl)) 
                  strncpy(rin_head[ 7],in_string,20);
  if (read_ok) if (read_ok=read_ctl_str(in_string,"ANT TYPE:",20,10,fd_ctl)) 
                  strncpy(rin_head[ 7]+20,in_string,20);

  // read the 12 real value elements of the control file,
  // and copy them to the header template
  if (read_ok) if (read_ok=read_ctl_dbl(appr_pos,"APPROX POSITION X:",11,fd_ctl)) 
                  sprintf(rin_head[ 8],"%14.4f",appr_pos[0]);
  if (read_ok) if (read_ok=read_ctl_dbl(appr_pos+1,"APPROX POSITION Y:",12,fd_ctl)) 
                  sprintf(rin_head[ 8]+14,"%14.4f",appr_pos[1]);
  if (read_ok) if (read_ok=read_ctl_dbl(appr_pos+2,"APPROX POSITION Z:",13,fd_ctl)) 
                 { sprintf(rin_head[ 8]+28,"%14.4f",appr_pos[2]);
                 // overwrite the null character with a blank
		 strncpy(rin_head[ 8]+42," ",1); }
  if (read_ok) if (read_ok=read_ctl_dbl(&in_dbl,"ANTENNA DELTA H:",14,fd_ctl)) 
                  sprintf(rin_head[ 9],"%14.4f",in_dbl);
  if (read_ok) if (read_ok=read_ctl_dbl(&in_dbl,"ANTENNA DELTA E:",15,fd_ctl)) 
                  sprintf(rin_head[ 9]+14,"%14.4f",in_dbl);
  if (read_ok) if (read_ok=read_ctl_dbl(&in_dbl,"ANTENNA DELTA N:",16,fd_ctl)) 
                 { sprintf(rin_head[ 9]+28,"%14.4f",in_dbl);
                 // overwrite the null character with a blank
		 strncpy(rin_head[ 9]+42," ",1); }
  if (read_ok) if (read_ok=read_ctl_dbl(c_n0_lim+0,"C/N0 for L1 snr flag=1:",17,fd_ctl)) 
                 { sprintf(rin_head[14]+5,"%4.0f",c_n0_lim[0]);
                 // overwrite the null character with a blank
		 strncpy(rin_head[ 14]+9," ",1); }
  if (read_ok) if (read_ok=read_ctl_dbl(c_n0_lim+1,"C/N0 for L1 snr flag=5:",18,fd_ctl)) 
                 { sprintf(rin_head[14]+15,"%4.0f",c_n0_lim[1]);
                 // overwrite the null character with a blank
		 strncpy(rin_head[ 14]+19," ",1); }
  if (read_ok) if (read_ok=read_ctl_dbl(c_n0_lim+2,"C/N0 for L1 snr flag=9:",19,fd_ctl)) 
                 { sprintf(rin_head[14]+25,"%4.0f",c_n0_lim[2]);
                 // overwrite the null character with a blank
		 strncpy(rin_head[ 14]+29," ",1); }
  if (read_ok) if (read_ok=read_ctl_dbl(c_n0_lim+3,"C/N0 for L2 snr flag=1:",20,fd_ctl)) 
                 { sprintf(rin_head[15]+5,"%4.0f",c_n0_lim[3]);
                 // overwrite the null character with a blank
		 strncpy(rin_head[ 15]+9," ",1); }
  if (read_ok) if (read_ok=read_ctl_dbl(c_n0_lim+4,"C/N0 for L2 snr flag=5:",21,fd_ctl)) 
                  sprintf(rin_head[15]+15,"%4.0f",c_n0_lim[4]);
                 // overwrite the null character with a blank
		 { strncpy(rin_head[ 15]+19," ",1); }
  if (read_ok) if (read_ok=read_ctl_dbl(c_n0_lim+5,"C/N0 for L2 snr flag=9:",22,fd_ctl)) 
                 { sprintf(rin_head[15]+25,"%4.0f",c_n0_lim[5]);
                 // overwrite the null character with a blank
		 strncpy(rin_head[ 15]+29," ",1); }

  if (file_is_open) fclose(fd_ctl);

  // return 1 if all readings were ok
  return read_ok;

}

