#include "decl.h"

void get_sat_coord(double coord[3],double t,struct ephemeris_record eph,int sat)

     // This function calculates the coordinates of a GPS satellite
     // at a given time in an Earth fixed coordinate system
     // using an ephemeris record.
     // See GPS SPS Signal Specification, AD 04-03, for ephemeris
     // definitions and algorithms.
     //
     // Input  t     : time in GPS seconds
     //        eph   : GPS ephemerides
     //        sat   : GPS satellite number
     //
     // Output coord : GPS satellite coordinates
 
{
  double mu=3.986005e14;
  double Omega_dot_e=7.2921151467e-5;
  double A,n_0,n,t_oe_f,t_k,M_k,E_k,nu_k,Phi_k,
       du_k,dr_k,di_k,u_k,r_k,i_k,xp_k,yp_k,Omega_k;

  A=pow(eph.sqtA[sat-1],2.0);
  n_0=pow(mu/pow(A,3.0),0.5);
  n=n_0+eph.dn[sat-1];

  t_oe_f=eph.t_oe[sat-1]+7.0*86400.0*eph.week_n[sat-1];

  // compensate for possible week roll over problems 
  if (t_oe_f > t+3.5*86400.0) t_oe_f = t_oe_f - 7.0*86400.0;
  if (t_oe_f < t-3.5*86400.0) t_oe_f = t_oe_f + 7.0*86400.0;


  t_k=t-t_oe_f;

  M_k=eph.M0[sat-1]+n*t_k;

  /* translate mean anomaly into eccentric anomaly */
  E_k=ecce_anom(M_k,eph.e[sat-1],3);

  /* translate eccentric anomaly into true anomaly */
  nu_k=atan2((pow(1-pow(eph.e[sat-1],2.0),0.5)*sin(E_k)),
                                (cos(E_k)-eph.e[sat-1]));

  /* Argument of latitude */
  Phi_k=nu_k+eph.omega[sat-1];

  /* Second harmonic perturbations */ 
  du_k=eph.C_us[sat-1]*sin(2*Phi_k)+eph.C_uc[sat-1]*cos(2*Phi_k); 
  dr_k=eph.C_rs[sat-1]*sin(2*Phi_k)+eph.C_rc[sat-1]*cos(2*Phi_k);
  di_k=eph.C_is[sat-1]*sin(2*Phi_k)+eph.C_ic[sat-1]*cos(2*Phi_k);

  u_k=Phi_k+du_k;                                   /* Corrected arg. of lat. */
  r_k=A*(1-eph.e[sat-1]*cos(E_k))+dr_k;           /* Corrected radius */
  i_k=eph.i_0[sat-1]+di_k+eph.i_dot[sat-1]*t_k; /* Corrected inclination */

  /* Sat pos. in orb. plane */
  xp_k=r_k*cos(u_k);
  yp_k=r_k*sin(u_k);

  Omega_k=eph.Omega_0[sat-1]+
       (eph.Omega_dot[sat-1]-Omega_dot_e)*t_k-
        Omega_dot_e*eph.t_oe[sat-1];
  
  /* Sat pos. in the Earth fixed coord system */
  coord[0]=xp_k*cos(Omega_k)-yp_k*cos(i_k)*sin(Omega_k);
  coord[1]=xp_k*sin(Omega_k)+yp_k*cos(i_k)*cos(Omega_k);
  coord[2]=yp_k*sin(i_k);

}



