#include "decl.h"

int read_command_line(int *time_interval_rinex,int *clock_tags,int *gps_only,int *day_break,
                   char rims_names[max_n_rims][80],char rinex_name[80],char run_by_name[20],
                   char rims_sta[5],int argc, char *argv[])

     // This function reads the program command line with its arguments, and
     // interpret the options controlling the program execution.
     //
     // Input  argc                : number of command line arguments
     //        argv                : command line arguments vector
     //
     // Output time_interval_rinex : rinex sampling time in seconds
     //        clock_tags          : (boolean) 1 if receiver clock offset in rinex file
     //        gps_only            : (boolean) 1 if only GPS data in rinex file
     //        day_break           : (boolean) 1 if new rinex file at day boudaries
     //        rims_names          : RIMS file name list
     //        rinex_name          : Rinex file name
     //        run_by_name         : program user name for rinex file header
     //        rims_sta            : RIMS station 5 character code
     //        read_command_line   : (boolean) 1 if reading ok

{
  int i,more_data_expected=1,echo_usage=0,reading_ok=1,n_rims_files=0,sl;
  int rims_sorting=0,rims_sta_found=0;
  char temp_rims_names[max_n_rims][80];


  // set default values for some outputs
  *time_interval_rinex=1;   // rinex sampling time interval = 1 s
  *clock_tags=0;            // no receiver clock offset written
  *gps_only=0;              // use all systems: GPS+GLONASS+GEO
  *day_break=0;             // no new rinex file at day boundary

  // default names for rinex file and program user: empty strings
  strcpy(rinex_name,"");
  strcpy(run_by_name,"");

  // Loop over the command line arguments to extract options.
  // For those options that contain more data than just the flag
  // the "more_data_expected" variable is set to a non-zero value.
  // When the next argument is read the "more_data_expected" value 
  // will guide the switch statement to associate the argument
  // with the right option
  for (i=1; i < argc; i++)
    {
    // input RIMS file names follow (-i flag not necessary if first option
    // since the variable is declared as more_data_expected=1 above)
    if      (strcmp(argv[i],"-i")==0) more_data_expected=1;
    // RIMS file name sorting activated
    else if (strcmp(argv[i],"-s")==0) {rims_sorting=1; more_data_expected=0;}
    // rinex sampling time interval follow
    else if (strcmp(argv[i],"-t")==0) more_data_expected=2;
    // receiver clock offset tag output activated
    else if (strcmp(argv[i],"-c")==0) {*clock_tags=1; more_data_expected=0;}
    // GPS only in rinex file activated
    else if (strcmp(argv[i],"-g")==0) {*gps_only=1;  more_data_expected=0;}
    // new rinex file at day boundary option activated
    else if (strcmp(argv[i],"-d")==0) {*day_break=1; more_data_expected=0;}
    // program user names follow
    else if (strcmp(argv[i],"-n")==0) more_data_expected=3;
    // RIMS station code follow
    else if (strcmp(argv[i],"-r")==0) more_data_expected=4;
    // output rinex file name follow
    else if (strcmp(argv[i],"-o")==0) more_data_expected=5;
    else
      // not a recognized option flag, (hopefully additional data)
      switch(more_data_expected)
	{
	// no more data expected for last option, something is wrong
        case 0:
          fprintf(stderr,"Illegal command line%s",lin_end);
	  echo_usage=1;
	  reading_ok=0;
          break;
	// -i last option, read RIMS file name
        case 1:
          n_rims_files++;
          // check that # names don't exceed the allocation
          if (n_rims_files<=max_n_rims)
            strncpy(temp_rims_names[n_rims_files-1],argv[i],79);
          // at first occasion of # names exceeding the limit: give a message  
          if (n_rims_files == max_n_rims+1)
	    {
            fprintf(stderr,"Maximum %d RIMS files allowed%s",max_n_rims,lin_end);
    	    reading_ok=0;
            }
          break;
	// -t last option, read rinex sampling time interval
        case 2:
          *time_interval_rinex=atoi(argv[i]);
          if (*time_interval_rinex<=0)
	    {
            fprintf(stderr,"Rinex time interval must be a positive integer%s",lin_end);
    	    reading_ok=0;
            }
          break;
	// -n last option, read program user name
        case 3:
   	  // name can be built from several command line arguments,
   	  // check the total name length so far (max 20 characters)
          sl=strlen(run_by_name);	      
          // if length > 18 then there is no more space, since
          // we separate the components by adding a blank
          if (sl<19)
	    {
	    // if first component
            if (sl==0) strncpy(run_by_name,argv[i],20);
            else       
              {
	      // add blank for separation
              strncpy(run_by_name+sl," ",1);
              // add new component, don't exceed 20 characters         
	      strncpy(run_by_name+sl+1,argv[i],20-sl-1);
              }
            }
          break;
	// -r last option, read a 5 character RIMS station code (e.g. A0001)
        case 4:
          strncpy(rims_sta,argv[i],5);
          rims_sta_found=1;
          break;
	// -o last option, read rinex file name
        case 5:
          strncpy(rinex_name,argv[i],79);
          break;
        }
    }

  // construct the final rims_names list of RIMS file name,
  // either from sorting or just copying temp_rims_names
  if (reading_ok) 
    name_sort(rims_names,temp_rims_names,n_rims_files,rims_sorting);

  if ((reading_ok)&&(n_rims_files==0))
    {
    fprintf(stderr,"No RIMS file names found%s",lin_end);
    echo_usage=1;
    }

  // if no RIMS station code has been given 
  // try to extract it from the first RIMS file name
  if ((reading_ok)&&(n_rims_files!=0)&&(!rims_sta_found))
    if (!extract_rims_station(rims_sta,rims_names[0]))
      {
      fprintf(stderr,"No RIMS station name found%s",lin_end);
      echo_usage=1;
      // mark that reading of the command line is not ok,
      // since RIMS station code is necessary
      reading_ok=0;
      }

  // if errors have occurred that might indicate a need for
  // information on the program usage; give some help
  if (echo_usage) print_usage();

  if (!reading_ok) return 0;
  else  return n_rims_files; // there are RIMS file names AND station code

}
