#include "decl.h"

int main(int argc, char *argv[])

     // This program reads RIMS Raw Measurement data file and writes the  
     // GNSS data to RINEX Observation files.
     // The program run is controlled by a set of command line options.
     // Guiding documents:
     // "EGNOS CCF Interface Control Document (CCF ICD)" with annexes 
     // "RINEX: The Receiver Independent Exchange Format Version 2.10"
     //    by Werner Gurtner, AIUB
     // "GPS SPS Signal Specification", (AD 04-03 in CCF ICD)

{
  int rinexf_open=0,read_status=-1,eph_initialized=0,time_interval_rinex,
      clock_tags,gps_only,day_break,n_rims,i,day_number=0;
  FILE *fd_rims,*fd_rinex;
  char rimsf_names[max_n_rims][80],rinexf_name[80],run_by_name[20],
       rims_station[5],rinex_station[4],rinex_header[18][81];
  unsigned char raw_message_bytes[max_bytes];
  struct ephemeris_record ephemerides;
  struct measurement_record GNSS_data;
  double c_n0_lim[6],approx_position[3];


  // extract data from the command line
  // stop execution if no RIMS file names are found (n_rims=0)
  if ((n_rims=read_command_line(&time_interval_rinex,&clock_tags,&gps_only,&day_break,
	          rimsf_names,rinexf_name,run_by_name,rims_station,argc,argv))==0) exit(0);

  // read data from the control file and create rinex file header template
  // stop execution if reading error
  if (!extract_station_data(rinex_header,rinex_station,c_n0_lim,approx_position,
                  time_interval_rinex,gps_only,run_by_name,rims_station)) exit(1);
	
  // set measurement status values =0, i.e. no valid data in the present GNSS_data
  GNSS_data=reset_data();

  // loop over all RIMS file names
  for (i=0; i < n_rims; i++)
    {
    // read_status=-1 at start of new rims file, =0 while reading, 1 when file ready
    if ((fd_rims=fopen(rimsf_names[i],"r")) == NULL)     {
      fprintf(stderr, "Cannot open input file: %s%s",rimsf_names[i],lin_end); read_status=1;}
    else read_status=-1;

    // To calculate the receiver clock offset for the first measurements
    // it is necessary to read ahaed to find some sets of complete ephemeris
    if ((clock_tags)&&(read_status==-1)&&(!eph_initialized))
      {
      // read until there are ephemerides for 5 GPS satellites
      ephemerides=init_ephemeris(fd_rims);
      // mark that initialisation has been done
      eph_initialized=1;
      // rewind RIMS file to be able to read measurements from the beginning
      rewind(fd_rims);
      }

    // while RIMS file not ready
    while (read_status<1) 
      {
      // read RIMS file until an ok raw measurement message is found,
      // if none is found set read_status=1
      read_raw_message(raw_message_bytes,&read_status,fd_rims);
      // if raw_message_bytes contains a valid message
      if (read_status<1) 
        {
	// if receiver clocks offset wanted: extract ephemeris data from message 
        if (clock_tags) 
          extract_ephemerides(&ephemerides,GNSS_data.GPS_secs,raw_message_bytes);
	// extract GNSS measurement data from message 
        GNSS_data=extract_measurements(GNSS_data,raw_message_bytes,gps_only);
        // check if data should be written to a rinex file,
        // i.e. if there are new writeable data and time fits sampling interval
        if ((GNSS_data.writeable)&&(!((long)GNSS_data.GPS_secs%time_interval_rinex))) 
          {
	  // check if a new rinex file is needed,
	  // i.e. if a new one wanted because of day boundary or if none yet opened
          if (((day_change(&day_number,GNSS_data.GPS_secs))&&(day_break))||(!rinexf_open))
            { 
            if (rinexf_open) fclose(fd_rinex);
	    mk_rinexf_name(rinexf_name,rinex_station,day_number,day_break,rinexf_open);
            if ((fd_rinex=fopen(rinexf_name,"w")) == NULL)     {
              // stop execution if rinex file can't be opened
              fprintf(stderr, "Cannot open output file: %s%s",rinexf_name,lin_end);
              fclose(fd_rims);
              exit(2);}
            rinexf_open=1;
            // write header of the new rinex file
            write_header(rinex_header,GNSS_data.GPS_secs,fd_rinex);
            }
          // write a rinex file record from the GNSS_data
          write_rinex_record(GNSS_data,ephemerides,approx_position,clock_tags,
                                                  c_n0_lim,gps_only,fd_rinex); 
          } 
        } 
      }

    if (!(fd_rims == NULL)) fclose(fd_rims);
    }

  if (rinexf_open) fclose(fd_rinex);
}

