#include "decl.h"

void write_rinex_record(struct measurement_record measurement_data,struct ephemeris_record GPS_eph,
         double approx_position[3],int clock_tags,double c_n0_lim[6],int gps_only,FILE *fd)

     // This function writes the measurement record for one 
     // point in time to the rinex file.
     // See "RINEX: The Receiver Independent Exchange Format Version 2.10"
     // by Werner Gurtner, AIUB, for definitions.
     //
     // Input  measurement_data : present measurement data
     //        GPS_eph          : present GPS ephemerides
     //        approx_position  : RIMS station position from control file
     //        clock_tags       : (boolean) 1 if receiver clock offset in rinex file
     //        c_n0_lim         : rinex snr value mapping limits
     //        gps_only         : (boolean) 1 if only GPS data in the rinex file
     //        GPS_secs         : time of first observation, GPS seconds
     //        fd_rinex         : rinex file

{
  int i,dat[6],n_valid_sats,dt_ok,n_lines;
  double dt_receiver;
  char sat_string[3*NSAT+1];
  char out_lin[80+1];
  char pad_string[40];

  
  // create a string of blank that will be used for filling a couple of strings
  for (i=0; i < 40; i++) pad_string[i]=' '; 

  // derive an estimate of the receiver clock offset from the nominal second tick
  // using ranges to GPS satellites (from ephemerides and station positions), 
  // measured code pseudo ranges, and satellite clock offsets 
  if (clock_tags) dt_ok=calc_dt(&dt_receiver,measurement_data.GPS_secs,GPS_eph,
                                         approx_position,measurement_data.GPS);

  // count the number of satellites that have been measured
  // and write their number to sat_string
  n_valid_sats=0;
  for (i=0; i < NSAT; i++) if (measurement_data.GPS.status[i]) 
                              sprintf(sat_string+3*(n_valid_sats++),"G%2d",i+1);
  if (!gps_only)
    {
    for (i=0; i < NSAT; i++)  if (measurement_data.GLONASS.status[i]) 
                                sprintf(sat_string+3*(n_valid_sats++),"R%2d",i+1);
    for (i=0; i < NSAT; i++)  if (measurement_data.GEO.status[i])
                                sprintf(sat_string+3*(n_valid_sats++),"S%2d",i+1);
    }
  // mark the end of sat_string
  sprintf(sat_string+3*n_valid_sats,"%c",'\0');

  // convert the GPS seconds of thisobservation to
  // date and time elements
  s2ymdhms(dat, (long)(measurement_data.GPS_secs+0.3));
  // write date to rinex file, add "0" if year is just one digit
  if ((dat[0]%100)<10) fprintf(fd," 0%1d%3d%3d",dat[0]%100,dat[1],dat[2]);
  else fprintf(fd," %2d%3d%3d",dat[0]%100,dat[1],dat[2]);
  // write time and number of satellites to rinex file
  fprintf(fd,"%3d%3d%11.7f  0%3d",dat[3],dat[4],(double)dat[5],n_valid_sats);

  // write list of the first 12 satellites to rinex file
  strncpy(out_lin,sat_string,36);
  // fill with blanks if less than 12 satellites
  if (n_valid_sats<12) strncat(out_lin,pad_string,3*(12-n_valid_sats));
  sprintf(out_lin+3*12,"%c",'\0');
  // clock offset wanted and successfully estimated; write it
  if ((clock_tags)&&(dt_ok)) sprintf(out_lin+3*12,"%12.9f",dt_receiver);
  fprintf(fd,"%s%s",out_lin,lin_end);

  // calculate how many lines needed to write the satellite number list
  n_lines=(int)((n_valid_sats-1)/12)+1;
  // write the remaining part of the satellite number list
  for (i=1; i < n_lines; i++)
    {
    strncpy(out_lin,pad_string,32);
    strncpy(out_lin+32,sat_string+i*36,36);
    strncpy(out_lin+68,"\0",1);
    fprintf(fd,"%s%s",out_lin,lin_end);
    }

  // write all valid GPS measurements
  write_GNSS_system(measurement_data.GPS,c_n0_lim,0,fd);
  // if wanted, write all valid GLONASS and GEO measurements
  if (!gps_only)
    {
    write_GNSS_system(measurement_data.GLONASS,c_n0_lim,1,fd);
    write_GNSS_system(measurement_data.GEO,c_n0_lim,2,fd);
    }

}





